@@ -15,6 +15,7 @@
 #include <netinet/in.h>
 #include <unistd.h>
 #include <limits.h>
+#include <sys/socket.h>
 
 /**
  * @file libknet.h
@@ -77,6 +78,7 @@ typedef uint16_t knet_node_id_t;
  * Where possible, set traffic priority to high.
  * On Linux this sets the TOS to INTERACTIVE (6),
  * see tc-prio(8) for more infomation
+ * A dscp value may be configured, see knet_handle_setprio_dscp.
  */
 
 #define KNET_LINK_FLAG_TRAFFICHIPRIO (1ULL << 0)
@@ -513,6 +515,30 @@ int knet_handle_enable_filter(knet_handle_t knet_h,
 
 int knet_handle_setfwd(knet_handle_t knet_h, unsigned int enabled);
 
+/**
+ * knet_handle_setprio_dscp
+ *
+ * @brief Use dscp for IP_TOS on socket to implement KNET_LINK_FLAG_TRAFFICHIPRIO
+ *
+ * knet_h   - pointer to knet_handle_t
+ *
+ * dscp     - dscp value to set on all new sockets
+ *
+ * This function must be called prior to configure knet links.
+ *
+ * It disables the use of IPTOS_LOWDELAY and uses the given dscp value in the
+ * IP header's TOS field instead.
+ *
+ * Setting dscp to 0 reverts to using IPTOS_LOWDELAY.
+ *
+ * @return
+ * knet_handle_setprio_dscp returns
+ * 0 on success
+ * -1 on error and errno is set.
+ */
+
+int knet_handle_setprio_dscp(knet_handle_t knet_h, uint8_t dscp);
+
 /**
  * knet_handle_enable_access_lists
  *
