@@ -96,8 +96,8 @@ static void test(void)
 
 	addrv6 = (struct sockaddr_in6 *)&addr;
 	addrv6->sin6_family = AF_INET6;
-	addrv6->sin6_addr.s6_addr16[0] = htons(0xfd00); /* fd00::1 */
-	addrv6->sin6_addr.s6_addr16[7] = htons(0x0001);
+	addrv6->sin6_addr.s6_addr32[0] = htonl(0xfd000000); /* fd00::1 */
+	addrv6->sin6_addr.s6_addr32[3] = htonl(0x00000001);
 	addrv6->sin6_port = htons(50000);
 
 	if (knet_addrtostr(&addr, sizeof(struct sockaddr_storage),
