@@ -974,7 +974,7 @@ void *_handle_recv_from_links_thread(void *data)
 	struct knet_mmsghdr msg[PCKT_RX_BUFS];
 	struct iovec iov_in[PCKT_RX_BUFS];
 #if defined(IP_PKTINFO) || defined(IPV6_PKTINFO)
-	unsigned char control_in[CMSG_SPACE(sizeof(struct in6_pktinfo))][PCKT_RX_BUFS];
+	unsigned char control_in[PCKT_RX_BUFS][CMSG_SPACE(sizeof(struct in6_pktinfo))];
 #endif
 
 	set_thread_status(knet_h, KNET_THREAD_RX, KNET_THREAD_STARTED);
@@ -993,7 +993,7 @@ void *_handle_recv_from_links_thread(void *data)
 		msg[i].msg_hdr.msg_iov = &iov_in[i];
 		msg[i].msg_hdr.msg_iovlen = 1;
 #if defined(IP_PKTINFO) || defined(IPV6_PKTINFO)
-		msg[i].msg_hdr.msg_control = &control_in[0][i];
+		msg[i].msg_hdr.msg_control = &control_in[i][0];
 		msg[i].msg_hdr.msg_controllen = CMSG_SPACE(sizeof(struct in6_pktinfo)); /* Largest of the two pktinfo structs */
 #endif
 	}
